@@ -1,4 +1,4 @@
-/*	$OpenBSD: kroute.c,v 1.52 2007/07/23 12:21:35 pyr Exp $ */
+/*	$OpenBSD: kroute.c,v 1.55 2007/10/17 20:52:42 claudio Exp $ */
 
 /*
  * Copyright (c) 2004 Esben Norby <norby@openbsd.org>
@@ -65,6 +65,8 @@ int	kr_redist_eval(struct kroute *, struct rroute *);
 void	kr_redistribute(struct kroute_node *);
 int	kroute_compare(struct kroute_node *, struct kroute_node *);
 int	kif_compare(struct kif_node *, struct kif_node *);
+int	kr_change_fib(struct kroute_node *, struct kroute *, int, int);
+int	kr_delete_fib(struct kroute_node *);
 
 struct kroute_node	*kroute_find(in_addr_t, u_int8_t);
 struct kroute_node	*kroute_matchgw(struct kroute_node *, struct in_addr);
@@ -86,7 +88,7 @@ int		protect_lo(void);
 u_int8_t	prefixlen_classful(in_addr_t);
 void		get_rtaddrs(int, struct sockaddr *, struct sockaddr **);
 void		if_change(u_short, int, struct if_data *);
-void		if_newaddr(u_short, struct sockaddr_in *, struct sockaddr_in *, 
+void		if_newaddr(u_short, struct sockaddr_in *, struct sockaddr_in *,
 		    struct sockaddr_in *);
 void		if_announce(void *);
 
@@ -164,18 +166,104 @@ kr_init(int fs)
 }
 
 int
-kr_change(struct kroute *kroute)
+kr_change_fib(struct kroute_node *kr, struct kroute *kroute, int krcount,
+    int action)
+{
+	int			 i;
+	struct kroute_node	*kn, *nkn;
+
+	if (action == RTM_ADD) {
+		/*
+		 * First remove all stale multipath routes.
+		 * This step must be skipped when the action is RTM_CHANGE
+		 * because it is already a single path route that will be
+		 * changed.
+		 */
+		for (kn = kr; kn != NULL; kn = nkn) {
+			for (i = 0; i < krcount; i++) {
+				if (kn->r.nexthop.s_addr ==
+				    kroute[i].nexthop.s_addr)
+					break;
+			}
+			nkn = kn->next;
+			if (i == krcount)
+				/* stale route */
+				if (kr_delete_fib(kn) == -1)
+					log_warnx("kr_delete_fib failed");
+			log_debug("kr_update_fib: before: %s%s",
+			    inet_ntoa(kn->r.nexthop),
+			    i == krcount ? " (deleted)" : "");
+		}
+	}
+
+	/*
+	 * now add or change the route
+	 */
+	for (i = 0; i < krcount; i++) {
+		/* nexthop within 127/8 -> ignore silently */
+		if ((kroute[i].nexthop.s_addr & htonl(IN_CLASSA_NET)) ==
+		    htonl(INADDR_LOOPBACK & IN_CLASSA_NET))
+			continue;
+
+		if (action == RTM_ADD && kr) {
+			for (kn = kr; kn != NULL; kn = kn->next) {
+				if (kn->r.nexthop.s_addr ==
+				    kroute[i].nexthop.s_addr)
+					break;
+			}
+
+			log_debug("kr_update_fib: after : %s%s",
+			     inet_ntoa(kroute[i].nexthop),
+			     kn == NULL ? " (added)" : "");
+
+			if (kn != NULL)
+				/* nexthop already present, skip it */
+				continue;
+		} else
+			/* modify first entry */
+			kn = kr;
+
+		/* send update */
+		if (send_rtmsg(kr_state.fd, action, &kroute[i]) == -1)
+			return (-1);
+
+		/* create new entry unless we are changing the first entry */
+		if (action == RTM_ADD)
+			if ((kn = calloc(1, sizeof(*kn))) == NULL)
+				fatal(NULL);
+
+		kn->r.prefix.s_addr = kroute[i].prefix.s_addr;
+		kn->r.prefixlen = kroute[i].prefixlen;
+		kn->r.nexthop.s_addr = kroute[i].nexthop.s_addr;
+		kn->r.flags = kroute[i].flags | F_OSPFD_INSERTED;
+		kn->r.ext_tag = kroute[i].ext_tag;
+		rtlabel_unref(kn->r.rtlabel);	/* for RTM_CHANGE */
+		kn->r.rtlabel = kroute[i].rtlabel;
+
+		if (action == RTM_ADD)
+			if (kroute_insert(kn) == -1) {
+				log_debug("kr_update_fib: cannot insert %s",
+				    inet_ntoa(kn->r.nexthop));
+				free(kn);
+			}
+		action = RTM_ADD;
+	}
+	return  (0);
+}
+
+int
+kr_change(struct kroute *kroute, int krcount)
 {
 	struct kroute_node	*kr;
 	int			 action = RTM_ADD;
 
 	kroute->rtlabel = rtlabel_tag2id(kroute->ext_tag);
 
-	if ((kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen)) !=
-	    NULL) {
-		if (!(kr->r.flags & F_KERNEL))
-			action = RTM_CHANGE;
-		else {	/* a non-ospf route already exists. not a problem */
+	kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen);
+
+	if (kr != NULL) {
+		if (kr->r.flags & F_KERNEL) {
+			/* a non-ospf route already exists. not a problem */
 			if (!(kr->r.flags & F_BGPD_INSERTED)) {
 				do {
 					kr->r.flags |= F_OSPFD_INSERTED;
@@ -194,45 +282,30 @@ kr_change(struct kroute *kroute)
 			 * - zero out ifindex (this is no longer relevant)
 			 */
 			action = RTM_CHANGE;
-			kr->r.flags = kroute->flags | F_OSPFD_INSERTED;
-			kr->r.ifindex = 0;
-			rtlabel_unref(kr->r.rtlabel);
-			kr->r.ext_tag = kroute->ext_tag;
-			kr->r.rtlabel = kroute->rtlabel;
-		}
+		} else if (kr->next == NULL)	/* single path OSPF route */
+			action = RTM_CHANGE;
 	}
 
-	/* nexthop within 127/8 -> ignore silently */
-	if ((kroute->nexthop.s_addr & htonl(IN_CLASSA_NET)) ==
-	    htonl(INADDR_LOOPBACK & IN_CLASSA_NET))
+	return (kr_change_fib(kr, kroute, krcount, action));
+}
+
+int
+kr_delete_fib(struct kroute_node *kr)
+{
+	if (!(kr->r.flags & F_OSPFD_INSERTED))
 		return (0);
 
-	/*
-	 * Ingnore updates that did not change the route.
-	 * Currently only the nexthop can change.
-	 */
-	if (kr && kr->r.nexthop.s_addr == kroute->nexthop.s_addr)
+	if (kr->r.flags & F_KERNEL) {
+		/* remove F_OSPFD_INSERTED flag, route still exists in kernel */
+		kr->r.flags &= ~F_OSPFD_INSERTED;
 		return (0);
+	}
 
-	if (send_rtmsg(kr_state.fd, action, kroute) == -1)
+	if (send_rtmsg(kr_state.fd, RTM_DELETE, &kr->r) == -1)
 		return (-1);
 
-	if (action == RTM_ADD) {
-		if ((kr = calloc(1, sizeof(struct kroute_node))) == NULL) {
-			log_warn("kr_change");
-			return (-1);
-		}
-		kr->r.prefix.s_addr = kroute->prefix.s_addr;
-		kr->r.prefixlen = kroute->prefixlen;
-		kr->r.nexthop.s_addr = kroute->nexthop.s_addr;
-		kr->r.flags = kroute->flags | F_OSPFD_INSERTED;
-		kr->r.ext_tag = kroute->ext_tag;
-		kr->r.rtlabel = kroute->rtlabel;
-
-		if (kroute_insert(kr) == -1)
-			free(kr);
-	} else
-		kr->r.nexthop.s_addr = kroute->nexthop.s_addr;
+	if (kroute_remove(kr) == -1)
+		return (-1);
 
 	return (0);
 }
@@ -240,30 +313,18 @@ kr_change(struct kroute *kroute)
 int
 kr_delete(struct kroute *kroute)
 {
-	struct kroute_node	*kr;
+	struct kroute_node	*kr, *nkr;
 
 	if ((kr = kroute_find(kroute->prefix.s_addr, kroute->prefixlen)) ==
 	    NULL)
 		return (0);
 
-	if (!(kr->r.flags & F_OSPFD_INSERTED))
-		return (0);
-
-	if (kr->r.flags & F_KERNEL) {
-		/* remove F_OSPFD_INSERTED flag, route still exists in kernel */
-		do {
-			kr->r.flags &= ~F_OSPFD_INSERTED;
-			kr = kr->next;
-		} while (kr);
-		return (0);
+	while (kr != NULL) {
+		nkr = kr->next;
+		if (kr_delete_fib(kr) == -1)
+			return (-1);
+		kr = nkr;
 	}
-
-	if (send_rtmsg(kr_state.fd, RTM_DELETE, kroute) == -1)
-		return (-1);
-
-	if (kroute_remove(kr) == -1)
-		return (-1);
-
 	return (0);
 }
 
@@ -279,6 +340,7 @@ void
 kr_fib_couple(void)
 {
 	struct kroute_node	*kr;
+	struct kroute_node	*kn;
 
 	if (kr_state.fib_sync == 1)	/* already coupled */
 		return;
@@ -287,7 +349,9 @@ kr_fib_couple(void)
 
 	RB_FOREACH(kr, kroute_tree, &krt)
 		if (!(kr->r.flags & F_KERNEL))
-			send_rtmsg(kr_state.fd, RTM_ADD, &kr->r);
+			for (kn = kr; kn != NULL; kn = kn->next) {
+				send_rtmsg(kr_state.fd, RTM_ADD, &kn->r);
+			}
 
 	log_info("kernel routing table coupled");
 }
@@ -296,13 +360,17 @@ void
 kr_fib_decouple(void)
 {
 	struct kroute_node	*kr;
+	struct kroute_node	*kn;
 
 	if (kr_state.fib_sync == 0)	/* already decoupled */
 		return;
 
-	RB_FOREACH(kr, kroute_tree, &krt)
+	RB_FOREACH(kr, kroute_tree, &krt) {
 		if (!(kr->r.flags & F_KERNEL))
-			send_rtmsg(kr_state.fd, RTM_DELETE, &kr->r);
+			for (kn = kr; kn != NULL; kn = kn->next) {
+				send_rtmsg(kr_state.fd, RTM_DELETE, &kn->r);
+			}
+	}
 
 	kr_state.fib_sync = 0;
 
@@ -611,7 +679,7 @@ kroute_remove(struct kroute_node *kr)
 			    inet_ntoa(kr->r.prefix), kr->r.prefixlen);
 			return (-1);
 		}
-	       	if (kr->next != NULL) {
+		if (kr->next != NULL) {
 			if (RB_INSERT(kroute_tree, &krt, kr->next) != NULL) {
 				log_warnx("kroute_remove failed to add %s/%u",
 				    inet_ntoa(kr->r.prefix), kr->r.prefixlen);
@@ -900,16 +968,17 @@ if_change(u_short ifindex, int flags, struct if_data *ifd)
 }
 
 void
-if_newaddr(u_short ifindex, struct sockaddr_in *ifa, struct sockaddr_in *mask, 
+if_newaddr(u_short ifindex, struct sockaddr_in *ifa, struct sockaddr_in *mask,
     struct sockaddr_in *brd)
-{   
+{
 	struct kif_node *kif;
 	struct kif_addr *ka;
-		     
+
 	if (ifa == NULL || ifa->sin_family != AF_INET)
 		return;
 	if ((kif = kif_find(ifindex)) == NULL) {
-		log_warnx("if_newaddr: corresponding if %i not found", ifindex);		return;
+		log_warnx("if_newaddr: corresponding if %i not found", ifindex);
+		return;
 	}
 	if ((ka = calloc(1, sizeof(struct kif_addr))) == NULL)
 		fatal("if_newaddr");
@@ -966,7 +1035,7 @@ send_rtmsg(int fd, int action, struct kroute *kroute)
 	bzero(&hdr, sizeof(hdr));
 	hdr.rtm_version = RTM_VERSION;
 	hdr.rtm_type = action;
-	hdr.rtm_flags = RTF_PROTO2;
+	hdr.rtm_flags = RTF_PROTO2|RTF_MPATH;
 	if (action == RTM_CHANGE)	/* force PROTO2 reset the other flags */
 		hdr.rtm_fmask = RTF_PROTO2|RTF_PROTO1|RTF_REJECT|RTF_BLACKHOLE;
 	hdr.rtm_seq = kr_state.rtseq++;	/* overflow doesn't matter */
@@ -1097,6 +1166,8 @@ fetchtable(void)
 	lim = buf + len;
 	for (next = buf; next < lim; next += rtm->rtm_msglen) {
 		rtm = (struct rt_msghdr *)next;
+		if (rtm->rtm_version != RTM_VERSION)
+			continue;
 		sa = (struct sockaddr *)(rtm + 1);
 		get_rtaddrs(rtm->rtm_addrs, sa, rti_info);
 
@@ -1277,6 +1348,8 @@ dispatch_rtmsg(void)
 	lim = buf + n;
 	for (next = buf; next < lim; next += rtm->rtm_msglen) {
 		rtm = (struct rt_msghdr *)next;
+		if (rtm->rtm_version != RTM_VERSION)
+			continue;
 
 		prefix.s_addr = 0;
 		prefixlen = 0;
@@ -1438,7 +1511,7 @@ add:
 				    " not found");
 				return (-1);
 			}
-			/* 
+			/*
 			 * last route is getting removed request the
 			 * ospf route from the RDE to insert instead
 			 */
