@@ -1,4 +1,4 @@
-/*	$OpenBSD: ospfe.c,v 1.57 2007/07/25 19:11:27 claudio Exp $ */
+/*	$OpenBSD: ospfe.c,v 1.60 2007/10/13 13:21:24 claudio Exp $ */
 
 /*
  * Copyright (c) 2005 Claudio Jeker <claudio@openbsd.org>
@@ -895,7 +895,7 @@ orig_rtr_lsa(struct area *area)
 	 * Set the E bit as soon as an as-ext lsa may be redistributed, only
 	 * setting it in case we redistribute something is not worth the fuss.
 	 */
-	if (oeconf->redistribute && (oeconf->options & OSPF_OPTION_E))
+	if (oeconf->redistribute && !area->stub)
 		lsa_rtr.flags |= OSPF_RTR_E;
 
 	border = (area_border_router(oeconf) != 0);
@@ -918,7 +918,7 @@ orig_rtr_lsa(struct area *area)
 
 	/* LSA header */
 	lsa_hdr.age = htons(DEFAULT_AGE);
-	lsa_hdr.opts = oeconf->options;		/* XXX */
+	lsa_hdr.opts = area_ospf_options(area);
 	lsa_hdr.type = LSA_TYPE_ROUTER;
 	lsa_hdr.ls_id = oeconf->rtr_id.s_addr;
 	lsa_hdr.adv_rtr = oeconf->rtr_id.s_addr;
@@ -982,7 +982,7 @@ orig_net_lsa(struct iface *iface)
 	else
 		lsa_hdr.age = htons(MAX_AGE);
 
-	lsa_hdr.opts = oeconf->options;		/* XXX */
+	lsa_hdr.opts = area_ospf_options(iface->area);
 	lsa_hdr.type = LSA_TYPE_NETWORK;
 	lsa_hdr.ls_id = iface->addr.s_addr;
 	lsa_hdr.adv_rtr = oeconf->rtr_id.s_addr;
@@ -1008,7 +1008,7 @@ ospfe_router_id(void)
 }
 
 void
-ospfe_fip_update(int type)
+ospfe_fib_update(int type)
 {
 	int	old = oe_nofib;
 
@@ -1068,7 +1068,7 @@ ospfe_demote_area(struct area *area, int active)
 		return;
 
 	bzero(&dmsg, sizeof(dmsg));
-	strlcpy(dmsg.demote_group, area->demote_group,  
+	strlcpy(dmsg.demote_group, area->demote_group,
 	sizeof(dmsg.demote_group));
 	dmsg.level = area->demote_level;
 	if (active)
@@ -1087,7 +1087,7 @@ ospfe_demote_iface(struct iface *iface, int active)
 		return;
 
 	bzero(&dmsg, sizeof(dmsg));
-	strlcpy(dmsg.demote_group, iface->demote_group,  
+	strlcpy(dmsg.demote_group, iface->demote_group,
 	sizeof(dmsg.demote_group));
 	if (active)
 		dmsg.level = -1;
